<?php

namespace Drupal\csv_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\file\Plugin\Field\FieldWidget\FileWidget;

/**
 * Plugin implementation of the 'file_generic' widget.
 *
 * @FieldWidget(
 *   id = "csv_file_generic",
 *   label = @Translation("Csv File"),
 *   field_types = {
 *     "csv_file"
 *   }
 * )
 */
class CsvFileWidget extends FileWidget {

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element = parent::formElement($items, $delta, $element, $form, $form_state);
    $element['#process'] = array_merge($element['#process'], [[static::class, 'process']]);
    return $element;
  }

  /**
   * Form API callback: Processes a file_generic field element.
   *
   * Expands the file_generic type to include the description and display
   * fields.
   *
   * This method is assigned as a #process callback in formElement() method.
   */
  public static function process($element, FormStateInterface $form_state, $form) {
    $item = $element['#value'];

    $element['settings'] = [
      '#type' => 'details',
      '#title' => t('Configuration'),
      '#open' => FALSE,
    ];

    $element['settings']['paging'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable paging'),
      '#default_value' => !empty($item['settings']['paging']) ? 1 : 0,
    ];

    $element['settings']['pageLength'] = [
      '#type' => 'select',
      '#title' => t('Page Length'),
      '#options' => [
        10 => 10,
        25 => 25,
        50 => 50,
        75 => 75,
        100 => 100,
      ],
      '#default_value' => !empty($item['settings']['pageLength']) ? $item['settings']['pageLength'] : 10,
      '#description' => t('The initial number of rows to display on a single page when using pagination.'),
    ];

    $element['settings']['lengthChange'] = [
      '#type' => 'checkbox',
      '#title' => t('Length Change'),
      '#default_value' => !empty($item['settings']['lengthChange']) ? 1 : 0,
      '#description' => t('When pagination is enabled, this option will control the display of an option for the end user to change the number of records to be shown per page.'),
    ];

    $element['settings']['searching'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable searching'),
      '#default_value' => !empty($item['settings']['searching']) ? 1 : 0,
    ];

    $element['settings']['responsive'] = [
      '#type' => 'select',
      '#title' => t('Responsive Option'),
      '#options' => [
        'childRow' => t('Child Row'),
        'horizontal_scroll' => t('Horizontal Scroll'),
      ],
      '#description' => t('The default "childRow" option, allows the display of hidden rows by clicking a plus button on each row.'),
      '#default_value' => !empty($item['settings']['responsive']) ? $item['settings']['responsive'] : 'childRow',
    ];

    return $element;

  }

  /**
   * Form API callback. Retrieves the value for the csv_file field element.
   *
   * This method is assigned as a #value_callback in formElement() method.
   */
  public static function value($element, $input, FormStateInterface $form_state) {
    $return = parent::value($element, $input, $form_state);

    if (empty($return['settings'])) {
      $return['settings'] = [
        'paging' => 1,
        'searching' => 1,
        'pageLength' => 10,
        'lengthChange' => 0,
        'responsive' => 'childRow',
      ];
    }
    else {
      // Some Drupal form elements convert integers to strings.
      // Assure integer values are saves as integers, not strings.
      $int_settings = [
        'paging',
        'searching',
        'pageLength',
        'lengthChange',
      ];
      foreach ($int_settings as $key) {
        if (array_key_exists($key, $return['settings'])) {
          $return['settings'][$key] = (int) $return['settings'][$key];
        }
      }
    }

    return $return;
  }

}
